#!/usr/bin/perl

# Script for searching for unused property keys
# Antonio García Domínguez
# $Id$
#
# Licensed under the same terms as Perl itself.
# Requires the 'xargs', 'find' and 'grep' command line utilities.

use strict;
use warnings;

# Arguments
if (scalar @ARGV == 0) {
    print STDERR "Usage: $0 [.properties files]\n";
    exit 1;
}
my @files = @ARGV;

# Dump all property keys in a key-position hash
my %keys     = ();
for my $file (@files)
{
    open(IF, "<$file")
	or die "$file couldn't be opened: $!";
    my @lines = <IF>;
    close(IF);

    for my $i (1..$#lines) 
    {
	next if $lines[$i] !~ /^\s*(\S+)\s*=/;
	my $key = $1;

	warn "$key has been repeated (previous position: $keys{$key}, current position: $file:$i)" 
	    if exists $keys{$key};
	$keys{$key} = "$file:$i";
    }
}

# Search for the key in all .java files under the current directory
foreach my $key (sort (keys(%keys)))
{
    my $position = $keys{$key};
    my $output   = `find -name '*.java' -type f | xargs grep -Rl $key`;
    print "Unused key: $key ($position)\n" if $output =~ /^\s*$/;
}

exit 0;
