package es.uca.xmleye.util;

/*
  Copyright (C) 2005-2008 Antonio García Domínguez
  
  This program is free software; you can redistribute it 
  and/or modify it under the terms of the GNU General Public 
  License as published by the Free Software Foundation; 
  either version 2 of the License, or (at your option) any 
  later version.
  
  This program is distributed in the hope that it will be 
  useful, but WITHOUT ANY WARRANTY; without even the implied 
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
  PURPOSE. See the GNU General Public License for more 
  details.
  
  You should have received a copy of the GNU General Public 
  License along with this program; if not, write to the Free 
  Software Foundation, Inc., 59 Temple Place, Suite 330, 
  Boston, MA 02111-1307 USA
*/

import es.uca.xmleye.util.XHTMLSearchKeyHighlighter;
import junit.framework.*;
import junit.textui.*;

/** 
    Tests de unidad para XHTMLSearchKeyHighlighter.
*/
public class XHTMLSearchKeyHighlighterTest extends TestCase 
{
  private XHTMLSearchKeyHighlighter marcador;

  /** Crea un nuevo caso de prueba. */
  public XHTMLSearchKeyHighlighterTest(String nombre) 
  {
    super(nombre);
  }

  /** Crea el conjunto de pruebas para JUnit. */
  public static Test suite() 
  {
    return new TestSuite(XHTMLSearchKeyHighlighterTest.class);
  }

  /** Lanza el caso de prueba mediante JUnit.*/
  public static void main(String[] args) 
  {
    TestRunner.run(suite());
  }

  /***** PRUEBAS ************************************/

  private final String docOriginal 
  = "<html><head><title>barra ka</title></head>"
  + "<body><barra>:foo barra sor abSORto SOR ka title :xor</barra></body></html>";
  private final String marcaAntes
  = "<font class=\"searchkey\">";
  private final String marcaDespues
  = "</font>";
  private final String DESTACA_REGEXP = marcaAntes + "$1" + marcaDespues;

  /**  Reemplazo con el caso más simple. */
  public void testReemplazarOmision() {
    final String salidaEsperada
      = "<html><head><title>barra ka</title></head>"
      + "<body><barra>:foo barra " + marcaAntes + "sor" + marcaDespues 
      + " ab" + marcaAntes + "SOR" + marcaDespues + "to " 
      + marcaAntes + "SOR" + marcaDespues 
      + " ka title :xor</barra></body></html>";

    assertTrue(salidaEsperada.equals(marcador.marcarApariciones(docOriginal,
								   "sor",
								   DESTACA_REGEXP,
								   false, false)));
  }

  /**  Palabra completa. */
  public void testReemplazarPalabraCompleta() {
    final String salidaEsperada
      = "<html><head><title>barra ka</title></head>"
      + "<body><barra>:foo barra " + marcaAntes + "sor" + marcaDespues 
      + " abSORto " 
      + marcaAntes + "SOR" + marcaDespues 
      + " ka title :xor</barra></body></html>";

    assertTrue(salidaEsperada.equals(marcador.marcarApariciones(docOriginal,
								   "sor",
								   DESTACA_REGEXP,
								   true,
								   false)));
  }


  /**  Coincidencia de mayúsculas/minúsculas. */
  public void testReemplazarMayusMinus() {
    final String salidaEsperada
      = "<html><head><title>barra ka</title></head>"
      + "<body><barra>:foo barra sor"
      + " ab" + marcaAntes + "SOR" + marcaDespues + "to " 
      + marcaAntes + "SOR" + marcaDespues 
      + " ka title :xor</barra></body></html>";

    assertTrue(salidaEsperada.equals(marcador.marcarApariciones(docOriginal,
								   "SOR",
								   DESTACA_REGEXP,
								   false,
								   true)));
  }

  /**  Coincidencia de mayúsculas/minúsculas y palabra completa. */
  public void testReemplazarPCompletaMayusMinus() {
    final String salidaEsperada
      = "<html><head><title>barra ka</title></head>"
      + "<body><barra>:foo barra sor"
      + " abSORto " 
      + marcaAntes + "SOR" + marcaDespues 
      + " ka title :xor</barra></body></html>";

    assertTrue(salidaEsperada.equals(marcador.marcarApariciones(docOriginal,
								   "SOR",
								   DESTACA_REGEXP,
								   true,
								   true)));
  }

  /**  El texto también existe dentro de &lt;head&gt;: no debería reemplazarse eso. */
  public void testReemplazarCabecera() {
    final String salidaEsperada
      = "<html><head><title>barra ka</title></head>"
      + "<body><barra>:foo barra sor abSORto SOR"
      + " " + marcaAntes + "ka" + marcaDespues + " title :xor</barra></body></html>";

    assertTrue(salidaEsperada.equals(marcador.marcarApariciones(docOriginal,
								   "ka",
								   DESTACA_REGEXP,
								   false,
								   false)));
  }

  /**  El texto también está dentro de &lt;head&gt; como un tag: no debería tocarse. */
  public void testReemplazarCabeceraConflictoTag() {
    final String salidaEsperada
      = "<html><head><title>barra ka</title></head>"
      + "<body><barra>:foo barra sor abSORto SOR"
      + " ka " + marcaAntes + "title" + marcaDespues + " :xor</barra></body></html>";

    assertTrue(salidaEsperada.equals(marcador.marcarApariciones(docOriginal,
								   "title",
								   DESTACA_REGEXP,
								   false,
								   false)));
  }

  /**  El texto también está en &lt;body&gt; como un tag: no debería tocarse. */
  public void testReplaceTagConflict() {
    final String salidaEsperada
      = "<html><head><title>barra ka</title></head>"
      + "<body><barra>:foo " + marcaAntes + "barra" + marcaDespues + " sor abSORto SOR"
      + " ka title :xor</barra></body></html>";


    assertTrue(salidaEsperada.equals(marcador.marcarApariciones(docOriginal,
								   "barra",
								   DESTACA_REGEXP,
								   false,
								   false)));
  }

  /** Cadena completa justo antes del final de un tag */
  public void testReemplazarPalabraCompletaTagInicial() {
    final String salidaEsperada
      = "<html><head><title>barra ka</title></head>"
      + "<body><barra>:foo barra sor abSORto SOR"
      + " ka title " + marcaAntes + ":xor" + marcaDespues + "</barra></body></html>";

    assertTrue(salidaEsperada.equals(marcador.marcarApariciones(docOriginal,
								   ":xor",
								   DESTACA_REGEXP,
								   true,
								   false)));
  }

  /** Palabras completas justo después de un tag */
  public void testReemplazarPalabraCompletaTagFinal() {
    final String salidaEsperada
      = "<html><head><title>barra ka</title></head>"
      + "<body><barra>" + marcaAntes + ":foo" + marcaDespues + " barra sor abSORto SOR"
      + " ka title :xor</barra></body></html>";

    assertTrue(salidaEsperada.equals(marcador.marcarApariciones(docOriginal,
								":foo",
								DESTACA_REGEXP,
								true,
								false)));
  }

  /** Accede a los recursos comunes necesarios para todas las pruebas. */
  protected void setUp()
  {
    marcador = new XHTMLSearchKeyHighlighter();
  }

  /** Libera los recursos asociados a las pruebas. */
  protected void tearDown() 
  {
    marcador = null;
  }

}
